#Controls for controlling Gas Jet experiment



import serial
import time
from itomUi import ItomUi
from itom import *

class QuantumComposer:
    def __init__(self):
        self.connection = serial.Serial(port="COM8", baudrate=9600, timeout=0.5)

    def writes(self, command):
        self.connection.write(command.encode())

    def readlines(self):
        line = None
        j=1
        while (line != "" and j<50):
            line = self.connection.readline().decode()
            line = line.strip("\n")
            j = j+1
            print(line)
            
    def channel(self, num):
        number = str(num)
        self.writes(":INST:NSEL " + number + "\r\n")

    def enableChannel(self, num):
        number = str(num)
        pulse_on = ":PULSE" + number + ":STAT ON\r\n"
        self.writes(pulse_on)

    def disableChannel(self, num):
        number = str(num)
        pulse_off = ":PULSE" + number + ":STAT OFF\r\n"
        self.writes(pulse_off)

    def run(self):
        system_on = ":PULSE0:STAT ON\r\n"
        self.writes(system_on)

    def stop(self):
        system_off = ":PULSE0:STAT OFF\r\n"
        self.writes(system_off)



class GasJetControls(ItomUi):
    def __init__(self):
        ItomUi.__init__(self, "GasJetUI.ui", type=ui.TYPEDIALOG)
        print("\nUI opened...")
        
        uiHandle = self.gui
        # Connect UI buttons here
        uiHandle.button1.connect("clicked()", self.testFunc)
        uiHandle.button2.connect("clicked()", self.runtest)
        uiHandle.go.connect("clicked()", self.automatedExperiment)
        uiHandle.manMove.connect("clicked()", self.manualMoving)

    #Test Functions
    def testFunc(self):
        print("Running test...")
        condition = self.gui.motor_on["checked"]
        print(condition)

        print("Done")

    def runtest(self):
        print("Running test on Quantum Composer...")
        qc = QuantumComposer()
        
        qc.enableChannel(1)
        qc.run()
        
        time.sleep(5)
        
        qc.stop()
        qc.disableChannel(1)

        del qc

        print("Done")

    #Actual Functions
    def manualMoving(self):
        print("Connecting to motor...")
        standa = actuator("Standa8SMC4USB", 0.01, 0 )
        
        print("Reading input parameters...")
        manBox = self.gui.manualBox
        manAngle = float(manBox.angle["text"])
        
        print("Done")

        
        manclockw = manBox.cwManButton["checked"]
        manaclockw = manBox.acwManButton["checked"]
        
        if manclockw == True:
            directedmanAngle = -manAngle
        if manaclockw == True:
            directedmanAngle = manAngle

        print("Moving...")
        standa.setPosRel(0, directedmanAngle)
        
        time.sleep(1)
        print("Done")
        del standa
        

    def automatedExperiment(self):
        #Initialise Quantum Compsoer
        print("\nInitialising Quantum Composer channels 1 and 2...")
        camera = dataIO("Ximea") #camera
        camera.startDevice()
        #setting camera parameters:
        camera.setParam("trigger_mode",2) #2 is rising edge
        camera.setAutoGrabbing(False)
        #quantum composer 
        qc = QuantumComposer()
        qc.enableChannel(1)
        qc.enableChannel(2)
        
        #Motor Connection
        print("Connecting to  motor...")
        standa = actuator("Standa8SMC4USB", 0.01, 0 )
        
        print("Done")



        
        #Get Parameters for Automation
        print("Reading input parameters...")
        autoBox = self.gui.automaticBox
        angleTotal = float(autoBox.totalAngle["text"]) #Total Angle to Rotate
        angleInc = float(autoBox.angleIncrement["text"]) #Angle Increments
        t = float(autoBox.timeInterval["text"]) #Time Between Stopping at Increments


        #Clockwise or Anticlockwise
        clockw = autoBox.cwButton["checked"]
        aclockw = autoBox.acwButton["checked"]
        
        if clockw == True:
            directedAngleInc = -angleInc
        if aclockw == True:
            directedAngleInc = angleInc

        print("Done")


        #Main Loop
        n = int(angleTotal/angleInc)
        for i in range (n):
            print("\nChecking motor status...")
            motor_status = self.gui.motor_on["checked"] #Enabled or disabled motor
            result = []
            d = dataObject()
            if motor_status == True:
                print("Motor is enabled")
                print("Running Quantum Composer...")
                qc.run() #Equivalent to pressing 'Run' button on QC
                #______________________
                #image saving code
                for j in range(0,20):
                    camera.acquire()
                    camera.getVal(d)
                    result.append(d.copy())
                 #save list of image
                saveIDC("image1.idc", {"result": result, "description": "sample"})
                 #______________________
                time.sleep(1)
                print("Rotating motor...")
                standa.setPosRel(0, directedAngleInc) #Rotation
                print("Total angle rotated : ", (i+1)*(-directedAngleInc))
                print("Waiting...")
                time.sleep(t) #Wait

            else:
                print("Motor is disabled")
                qc.stop()
                camera.closeDevice()
        
                break
                

        print("\nFinishing...")
        del standa
        del qc
        del camera
        
        print("Done")




UIinstance = GasJetControls()
UIinstance.show(1)
del UIinstance
print("UI closed")